// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

//
// This file was autogenerated by a tool.
// Do not modify it.
//

namespace Microsoft.Azure.Batch
{
    using Models = Microsoft.Azure.Batch.Protocol.Models;
    using System;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Contains utilization and resource usage statistics for the lifetime of a pool.
    /// </summary>
    public partial class PoolStatistics : IPropertyMetadata
    {
        private readonly DateTime lastUpdateTime;
        private readonly ResourceStatistics resourceStatistics;
        private readonly DateTime startTime;
        private readonly string url;
        private readonly UsageStatistics usageStatistics;

        #region Constructors

        internal PoolStatistics(Models.PoolStatistics protocolObject)
        {
            this.lastUpdateTime = protocolObject.LastUpdateTime;
            this.resourceStatistics = UtilitiesInternal.CreateObjectWithNullCheck(protocolObject.ResourceStats, o => new ResourceStatistics(o).Freeze());
            this.startTime = protocolObject.StartTime;
            this.url = protocolObject.Url;
            this.usageStatistics = UtilitiesInternal.CreateObjectWithNullCheck(protocolObject.UsageStats, o => new UsageStatistics(o).Freeze());
        }

        #endregion Constructors

        #region PoolStatistics

        /// <summary>
        /// Gets the time at which the statistics were last updated. All statistics are limited to the range between <see 
        /// cref="StartTime"/> and this value.
        /// </summary>
        public DateTime LastUpdateTime
        {
            get { return this.lastUpdateTime; }
        }

        /// <summary>
        /// Gets statistics related to resource consumption by compute nodes in the pool, such as average CPU utilization.
        /// </summary>
        public ResourceStatistics ResourceStatistics
        {
            get { return this.resourceStatistics; }
        }

        /// <summary>
        /// Gets the start time of the time range covered by the statistics.
        /// </summary>
        public DateTime StartTime
        {
            get { return this.startTime; }
        }

        /// <summary>
        /// Gets the URL for the statistics.
        /// </summary>
        public string Url
        {
            get { return this.url; }
        }

        /// <summary>
        /// Gets statistics related to pool usage, such as the amount of core-time used.
        /// </summary>
        public UsageStatistics UsageStatistics
        {
            get { return this.usageStatistics; }
        }

        #endregion // PoolStatistics

        #region IPropertyMetadata

        bool IModifiable.HasBeenModified
        {
            //This class is compile time readonly so it cannot have been modified
            get { return false; }
        }

        bool IReadOnly.IsReadOnly
        {
            get { return true; }
            set
            {
                // This class is compile time readonly already
            }
        }

        #endregion // IPropertyMetadata
    }
}