// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.Batch.Models
{
    using System.Linq;

    /// <summary>
    /// Contains information about an Azure Batch account.
    /// </summary>
    [Microsoft.Rest.Serialization.JsonTransformation]
    public partial class BatchAccount : Resource
    {
        /// <summary>
        /// Initializes a new instance of the BatchAccount class.
        /// </summary>
        public BatchAccount() { }

        /// <summary>
        /// Initializes a new instance of the BatchAccount class.
        /// </summary>
        /// <param name="coreQuota">The core quota for this Batch
        /// account.</param>
        /// <param name="poolQuota">The pool quota for this Batch
        /// account.</param>
        /// <param name="activeJobAndJobScheduleQuota">The active job and job
        /// schedule quota for this Batch account.</param>
        /// <param name="id">The id of the resource</param>
        /// <param name="name">The name of the resource</param>
        /// <param name="type">The type of the resource</param>
        /// <param name="location">The location of the resource</param>
        /// <param name="tags">The tags of the resource</param>
        /// <param name="accountEndpoint">The endpoint used by this account to
        /// interact with the Batch services.</param>
        /// <param name="provisioningState">The provisioned state of the
        /// resource. Possible values include: 'Invalid', 'Creating',
        /// 'Deleting', 'Succeeded', 'Failed', 'Cancelled'</param>
        /// <param name="autoStorage">The properties and status of any auto
        /// storage account associated with the account.</param>
        public BatchAccount(int coreQuota, int poolQuota, int activeJobAndJobScheduleQuota, string id = default(string), string name = default(string), string type = default(string), string location = default(string), System.Collections.Generic.IDictionary<string, string> tags = default(System.Collections.Generic.IDictionary<string, string>), string accountEndpoint = default(string), ProvisioningState? provisioningState = default(ProvisioningState?), AutoStorageProperties autoStorage = default(AutoStorageProperties))
            : base(id, name, type, location, tags)
        {
            AccountEndpoint = accountEndpoint;
            ProvisioningState = provisioningState;
            AutoStorage = autoStorage;
            CoreQuota = coreQuota;
            PoolQuota = poolQuota;
            ActiveJobAndJobScheduleQuota = activeJobAndJobScheduleQuota;
        }

        /// <summary>
        /// Gets the endpoint used by this account to interact with the Batch
        /// services.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "properties.accountEndpoint")]
        public string AccountEndpoint { get; private set; }

        /// <summary>
        /// Gets or sets the provisioned state of the resource. Possible
        /// values include: 'Invalid', 'Creating', 'Deleting', 'Succeeded',
        /// 'Failed', 'Cancelled'
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "properties.provisioningState")]
        public ProvisioningState? ProvisioningState { get; set; }

        /// <summary>
        /// Gets or sets the properties and status of any auto storage account
        /// associated with the account.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "properties.autoStorage")]
        public AutoStorageProperties AutoStorage { get; set; }

        /// <summary>
        /// Gets or sets the core quota for this Batch account.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "properties.coreQuota")]
        public int CoreQuota { get; set; }

        /// <summary>
        /// Gets or sets the pool quota for this Batch account.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "properties.poolQuota")]
        public int PoolQuota { get; set; }

        /// <summary>
        /// Gets or sets the active job and job schedule quota for this Batch
        /// account.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "properties.activeJobAndJobScheduleQuota")]
        public int ActiveJobAndJobScheduleQuota { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="Microsoft.Rest.ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (this.AutoStorage != null)
            {
                this.AutoStorage.Validate();
            }
        }
    }
}
