// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
// 
// Code generated by Microsoft (R) AutoRest Code Generator 0.17.0.0
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.MachineLearning.CommitmentPlans.Models
{
    using System.Linq;

    /// <summary>
    /// Represents historical information about usage of the Azure resources
    /// associated with a commitment plan.
    /// </summary>
    public partial class PlanUsageHistory
    {
        /// <summary>
        /// Initializes a new instance of the PlanUsageHistory class.
        /// </summary>
        public PlanUsageHistory() { }

        /// <summary>
        /// Initializes a new instance of the PlanUsageHistory class.
        /// </summary>
        /// <param name="planDeletionOverage">Overage incurred as a result of
        /// deleting a commitment plan.</param>
        /// <param name="planMigrationOverage">Overage incurred as a result of
        /// migrating a commitment plan from one SKU to another.</param>
        /// <param name="planQuantitiesAfterUsage">Included quantities
        /// remaining after usage against the commitment plan's associated
        /// resources was calculated.</param>
        /// <param name="planQuantitiesBeforeUsage">Included quantities
        /// remaining before usage against the commitment plan's associated
        /// resources was calculated.</param>
        /// <param name="planUsageOverage">Usage against the commitment plan's
        /// associated resources which was not covered by included quantities
        /// and is therefore overage.</param>
        /// <param name="usage">Usage against the commitment plan's associated
        /// resources.</param>
        /// <param name="usageDate">The date of usage, in ISO 8601
        /// format.</param>
        public PlanUsageHistory(System.Collections.Generic.IDictionary<string, double?> planDeletionOverage = default(System.Collections.Generic.IDictionary<string, double?>), System.Collections.Generic.IDictionary<string, double?> planMigrationOverage = default(System.Collections.Generic.IDictionary<string, double?>), System.Collections.Generic.IDictionary<string, double?> planQuantitiesAfterUsage = default(System.Collections.Generic.IDictionary<string, double?>), System.Collections.Generic.IDictionary<string, double?> planQuantitiesBeforeUsage = default(System.Collections.Generic.IDictionary<string, double?>), System.Collections.Generic.IDictionary<string, double?> planUsageOverage = default(System.Collections.Generic.IDictionary<string, double?>), System.Collections.Generic.IDictionary<string, double?> usage = default(System.Collections.Generic.IDictionary<string, double?>), System.DateTime? usageDate = default(System.DateTime?))
        {
            PlanDeletionOverage = planDeletionOverage;
            PlanMigrationOverage = planMigrationOverage;
            PlanQuantitiesAfterUsage = planQuantitiesAfterUsage;
            PlanQuantitiesBeforeUsage = planQuantitiesBeforeUsage;
            PlanUsageOverage = planUsageOverage;
            Usage = usage;
            UsageDate = usageDate;
        }

        /// <summary>
        /// Gets or sets overage incurred as a result of deleting a commitment
        /// plan.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "planDeletionOverage")]
        public System.Collections.Generic.IDictionary<string, double?> PlanDeletionOverage { get; set; }

        /// <summary>
        /// Gets or sets overage incurred as a result of migrating a
        /// commitment plan from one SKU to another.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "planMigrationOverage")]
        public System.Collections.Generic.IDictionary<string, double?> PlanMigrationOverage { get; set; }

        /// <summary>
        /// Gets or sets included quantities remaining after usage against the
        /// commitment plan's associated resources was calculated.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "planQuantitiesAfterUsage")]
        public System.Collections.Generic.IDictionary<string, double?> PlanQuantitiesAfterUsage { get; set; }

        /// <summary>
        /// Gets or sets included quantities remaining before usage against
        /// the commitment plan's associated resources was calculated.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "planQuantitiesBeforeUsage")]
        public System.Collections.Generic.IDictionary<string, double?> PlanQuantitiesBeforeUsage { get; set; }

        /// <summary>
        /// Gets or sets usage against the commitment plan's associated
        /// resources which was not covered by included quantities and is
        /// therefore overage.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "planUsageOverage")]
        public System.Collections.Generic.IDictionary<string, double?> PlanUsageOverage { get; set; }

        /// <summary>
        /// Gets or sets usage against the commitment plan's associated
        /// resources.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "usage")]
        public System.Collections.Generic.IDictionary<string, double?> Usage { get; set; }

        /// <summary>
        /// Gets or sets the date of usage, in ISO 8601 format.
        /// </summary>
        [Newtonsoft.Json.JsonProperty(PropertyName = "usageDate")]
        public System.DateTime? UsageDate { get; set; }

    }
}
