// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Search.Models
{
    using System;
    using System.Linq;
    using System.Collections.Generic;
    using Newtonsoft.Json;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Microsoft.Rest.Azure;

    /// <summary>
    /// Flexibly separates text into terms via a regular expression pattern.
    /// This analyzer is implemented using Apache Lucene.
    /// <see href="http://lucene.apache.org/core/4_10_3/analyzers-common/org/apache/lucene/analysis/miscellaneous/PatternAnalyzer.html" />
    /// </summary>
    [JsonObject("#Microsoft.Azure.Search.PatternAnalyzer")]
    public partial class PatternAnalyzer : Analyzer
    {
        /// <summary>
        /// Initializes a new instance of the PatternAnalyzer class.
        /// </summary>
        public PatternAnalyzer() { }

        /// <summary>
        /// Initializes a new instance of the PatternAnalyzer class.
        /// </summary>
        public PatternAnalyzer(string name, bool? lowerCaseTerms = default(bool?), string pattern = default(string), RegexFlags flags = default(RegexFlags), IList<string> stopwords = default(IList<string>))
            : base(name)
        {
            LowerCaseTerms = lowerCaseTerms;
            Pattern = pattern;
            Flags = flags;
            Stopwords = stopwords;
        }

        /// <summary>
        /// Gets or sets a value indicating whether terms should be
        /// lower-cased. Default is true.
        /// </summary>
        [JsonProperty(PropertyName = "lowercase")]
        public bool? LowerCaseTerms { get; set; }

        /// <summary>
        /// Gets or sets a regular expression pattern to match token
        /// separators. Default is an expression that matches one or more
        /// whitespace characters.
        /// </summary>
        [JsonProperty(PropertyName = "pattern")]
        public string Pattern { get; set; }

        /// <summary>
        /// Gets or sets regular expression flags.
        /// </summary>
        [JsonProperty(PropertyName = "flags")]
        public RegexFlags Flags { get; set; }

        /// <summary>
        /// Gets or sets a list of stopwords.
        /// </summary>
        [JsonProperty(PropertyName = "stopwords")]
        public IList<string> Stopwords { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public override void Validate()
        {
            base.Validate();
        }
    }
}
